/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite s_HBOffs_HBOff0,
 * Boston, MA 02111-1s_HBOff07, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include "TFOptions.h"
#include "HeightFieldGenRandom.h"
#include "GlobalTrace.h"
#include "terraform.h"				// defines version number

extern int readhf (HeightField **HF, char *fname);



/*
 *  process_app_args: process the application-related command line arguments 
 */
void TFOptions::processAppArgs (HeightField **HF,int argc, char **argv)
{
	int			rc;

	for (int i=1; i<argc; i++)
		{
		if (argv[i] &&
	            (!strcmp(argv[i],"-f")||!strcmp(argv[i],"--file")) )
			{
			if (argc >= i+1 && argv[i+1])
				{
				if ((rc=readhf (HF, argv[++i])) < 0)
					{
					printf ("File read failed (rc=%d) for file [%s] ... Exiting\n", rc, argv[i]);
					exit (-1);
					}
				}
			else
				{
				printf ("Error: file argument for %s <file> missing ... Exiting\n", argv[i]);
				exit (-1);
				}
			}
		else
		if (!strcmp (argv[i], "-g") || !strcmp (argv[i], "--generate"))
			{
			*HF = new HeightField ();
			HeightFieldGenRandom::generate (*HF, 400);
			}
#ifdef NEVER
		else
		if (!strcmp (argv[i], "-cli"))
			do_gui=FALSE;
#endif
		else 

		// dummy checks we do in process_global_args
		if (!strcmp (argv[i], "-dbg") || !strcmp (argv[i], "--debug"))
			i++;
		else
		if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--verbose"))
			;
		else
			{
			printf ("Error: argument [%s] not recognized ... \n", 
				argv[i]);
			help ();
			exit (0);
			}
		}
} 


/*
 *  process_global_args: process the global command line arguments 
 */
void TFOptions::processGlobalArgs (int argc, char **argv)
{
	// see if we should set a certain trace level 
	for (int i=1; i<argc; i++)
		{
		if (!strcmp (argv[i], "-dbg") || !strcmp (argv[i], "--debug"))
			{
			if (argc >= i+2)	// 1 for count-to-index, 1 for next arg
				GlobalTrace::setTraceLevel (atoi (argv[++i]));
			else
				{
				printf ("Error: traceLevel argument missing for %s ... Exiting\n", argv[i]);
				exit (1);
				}
			}
		else
		if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--verbose"))
			GlobalTrace::setTraceLevel (GlobalTrace::TRACE_VERBOSE);
		else
		if (!strcmp (argv[i], "-version") || !strcmp(argv[i], "--version"))
			{
			printf ("terraform %d.%d.%d\n", TF_MAJOR_VERSION, 
				TF_MINOR_VERSION, TF_MICRO_VERSION);
			exit (0);
			}
		else
		if (!strcmp (argv[i], "--help") || !strcmp (argv[i], "-?"))
			{
			help ();
			exit (0);
			}

		}
}


/*
 * help: display recognized command line options 
 */
void TFOptions::help ()
{
	printf ("terraform %d.%d.%d\n", TF_MAJOR_VERSION, TF_MINOR_VERSION, 
		TF_MICRO_VERSION);
	printf ("  -dbg, --debug <level>		set debug to the specified level\n");
	printf ("  -f, --file <file>		read Height Field <file> upon startup\n");
	printf ("  -g, --generate 		generate a Height Field upon startup\n");
	printf ("  -version, --version		print version\n");
	printf ("  -v, --verbose			verbose mode\n");
	printf ("  -?, --help			print help and exit\n");
	printf ("\n");
}


