/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite s_HBOffs_HBOff0,
 * Boston, MA 02111-1s_HBOff07, USA.
 */


#include "TFPreviewDialog.h"
#include "GuiColormap.h"
#include "GuiColormapBands.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFPreviewDialog::TFPreviewDialog (HeightField *HF, HeightFieldDraw *HFD, 
				  char *winTitle, char *paramTitle,
				  int tableX, int tableY) 
	        : GuiDialogPreview (winTitle, paramTitle, tableX, tableY),
		  d_vBoxOptions (TRUE, 5)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFPreviewDialog::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFPreviewDialog::setHFobjs");

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFPreviewDialog\n");

	d_HFPreview = NULL;
	d_HFDPreview = NULL;
	d_HFOPreview = NULL;
	d_HFO = NULL;
}


/*
 *  destructor: clean up 
 */
TFPreviewDialog::~TFPreviewDialog ()
{
	if (d_HFO)
		delete d_HFO;
	if (d_HFPreview)
		delete d_HFPreview;
	if (d_HFDPreview)
		delete d_HFDPreview;
	if (d_HFOPreview)
		delete d_HFOPreview;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFPreviewDialog\n");
}


/*
 *  setHBobjs: set the HF objects we operate on and generate a scaled 
 * 	down version for the preview 
 */
void TFPreviewDialog::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	GuiColormap 	*cMap = HFD->getColormap();

	SanityCheck::bailout ((!HF), "HF==NULL", "TFPreviewDialog::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFPreviewDialog::setHFobjs");

	if (d_HFO)
		delete d_HFO;
	if (d_HFPreview)
		delete d_HFPreview;
	if (d_HFDPreview)
		delete d_HFDPreview;
	if (d_HFOPreview)
		delete d_HFOPreview;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
	d_HFPreview = d_HFO->hfResize (100, 100);
	d_HFDPreview = new HeightFieldDraw (d_HFPreview, this->d_drawArea);
	d_HFDPreview->setMode ("0");
	d_HFDPreview->setColormap (cMap);
	d_HFOPreview = new HeightFieldOps (d_HFPreview);
	d_HFPreview->backup ();
}


/*
 *  (virtual) checkboxCallbackPreview: process the 'use preview' checkbox 
 */
void TFPreviewDialog::checkboxCallbackPreview ()
{
	if (!d_cbUsePreview.get_active())
		{
		d_previewFrame->set_sensitive (FALSE);
		d_drawArea->set_sensitive (FALSE);	
		this->clearPreview ();
		}
	else
		{
		SanityCheck::bailout ((!d_HFDPreview), "HFDPreview==NULL", 
				"GuiDialogPreview::checkboxCallbackPreview");
		d_previewFrame->set_sensitive (TRUE);
		d_drawArea->set_sensitive (FALSE);	
		d_HFDPreview->draw ();
		}
}


