/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "terraform.h"

#include "math.h"
#include "TFGui.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiDialogOAC.h"
#include "HeightField.h"
#include "HeightFieldGenRandom.h"
#include "HeightFieldIO.h"
#include "MathGauss.h"
#include "TFOptions.h"


//
// instantiate static data members
//
const int 	GuiDialogOAC::s_HBOff=3;
const int 	GuiDialogOAC::s_VBOff=3;
const int 	MathGauss::s_gaussNRand=4;
const double 	MathGauss::s_maxRand=pow(2.0, 15.0)-1.0;
double 		MathGauss::s_gaussAdd=0.0;
double 		MathGauss::s_gaussFac=0.0;
int 		GlobalTrace::s_traceLevel=0;
bool 		TFOptions::s_fillSea=TRUE;
bool 		TFOptions::s_fastHalfRes=TRUE;
bool 		TFOptions::s_fastHalfYscale=TRUE;
bool 		TFOptions::s_fastZrot=FALSE;
int 		TFOptions::s_scale=1;
int 		TFOptions::s_syncRate=250;
int 		TFOptions::s_resolution=10;
float 		TFOptions::s_yscale=0.33;
float 		TFOptions::s_sealevel=0.33;

int readhf (HeightField **HF, char *fname);



/*
 *  readhf: create a new HF object and then read into it
 */
int readhf (HeightField **HF, char *fname)
{
	int		rc=0;
	char		*ext;
	HeightFieldIO	*HFIO=NULL;

	SanityCheck::bailout ((*HF), "*HF!=NULL", "readhf()");
	SanityCheck::bailout ((!fname), "fname==NULL", "readhf()");
	SanityCheck::bailout ((!strlen(fname)), "strlen(fname)==0", "readhf()");

	*HF = new HeightField ();
	HFIO = new HeightFieldIO (fname);

	ext = HFIO->getExtension();
	if (!strcmp(ext,"png") || !strcmp(ext,"tiff") || !strcmp(ext,"tif"))
		{
		delete HFIO;
		printf ("Sorry -- PNG and TIFF read not supported from command line\n");
		delete *HF;
		*HF = NULL;
		return 1;
		}

	rc = HFIO->read (*HF);
	if (rc)
		delete *HF;
	HFIO->close ();
	//delete HFIO;		// XXX this causes a segmentation fault (why??)

	return (rc);
}


/*
 *  main: run the terraform program 
 */
int main (int argc, char *argv[])
{
	int		do_gui=TRUE;
	HeightField	*HF=NULL;

	// process command line arguments 
	TFOptions::processGlobalArgs (argc, argv);
	TFOptions::processAppArgs (&HF, argc, argv);

	// use graphical user interface
	if (do_gui)
		{
		Gtk_Main	gtkMain (&argc, &argv);
		TFGui	*tfGui;

		tfGui = new TFGui (&gtkMain, argc, argv);
		tfGui->show ();
		if (HF)
			tfGui->setHeightField (HF);
		gtkMain.run();
		}
/*
	// use command line interface
	else
		{
		if (!(HFP = new HeightFieldParser (verbose)))
			{
			cerr << "Error instantiating HeightFieldParser ... Exiting\n";
			exit (NOGOOD);
			}
		HFP->P_Parse ();
		delete HFP;
		exit (1);
		}
*/

	exit (0);
}
